//
//  ViewController.m
//  AppnextNativeAdsSDKTest
//
//  Created by Eran Mausner on 29/08/2016.
//  Copyright © 2016 Appnext. All rights reserved.
//

#import "ViewController.h"
#import "AppnextNativeAdsTestTableViewCell.h"
#import <AppnextNativeAdsSDK/AppnextNativeAdsSDK.h>
#import "UIView+Toast.h"

@interface ViewController ()<UITableViewDataSource, UITableViewDelegate, AppnextNativeAdsRequestDelegate, AppnextNativeAdOpenedDelegate, AppnextNativeAdsTestTableViewCellDelegate>

@property (nonatomic, strong) AppnextNativeAdsSDKApi *api;
@property (nonatomic, strong) IBOutlet UITextField *countField;
@property (nonatomic, strong) IBOutlet UITextView *eventsTextView;
@property (nonatomic, strong) IBOutlet UITableView *adsTableView;
@property (nonatomic, strong) NSArray<AppnextAdData *> *adsArray;

@end

@implementation ViewController

- (void) viewDidLoad
{
    [super viewDidLoad];
    self.adsArray = [NSArray array];
    self.countField.text = @"10";
    self.api = [[AppnextNativeAdsSDKApi alloc] initWithPlacementID:@"b5c36d44-60b0-44e0-b489-683782e0ae2b"];
}

#pragma mark - Actions

- (IBAction) loadAds:(id)sender
{
    [self.countField resignFirstResponder];
    AppnextNativeAdsRequest *request = [[AppnextNativeAdsRequest alloc] init];
    NSUInteger count = 1;
    if (self.countField.text.length)
    {
        count = self.countField.text.integerValue;
    }
    request.count = count;
    request.creativeType = ANCreativeTypeManaged;
    [self.api loadAds:request withRequestDelegate:self];
    [self showActivity];
}

#pragma mark - Internal

- (void) showActivity
{
    self.view.userInteractionEnabled = NO;
    [self.view makeToastActivity];
}

- (void) hideActivity
{
    self.view.userInteractionEnabled = YES;
    [self.view hideToastActivity];
}

#pragma mark - AppnextNativeAdsRequestDelegate

- (void) onAdsLoaded:(NSArray<AppnextAdData *> *)ads forRequest:(AppnextNativeAdsRequest *)request
{
    [self hideActivity];

    NSString *description = [NSString stringWithFormat:@"onAdsLoaded:\r\n"];
    NSLog(@"%@", description);
    self.eventsTextView.text = [NSString stringWithFormat:@"%@%@", self.eventsTextView.text, description];
    self.adsArray = [NSArray arrayWithArray:ads];
    [self.adsTableView reloadData];
}

- (void) onError:(NSString *)error forRequest:(AppnextNativeAdsRequest *)request
{
    [self hideActivity];

    NSString *description = [NSString stringWithFormat:@"onError:forRequest: %@\r\n", error];
    NSLog(@"%@", description);
    self.eventsTextView.text = [NSString stringWithFormat:@"%@%@", self.eventsTextView.text, description];
    [self.view makeToast:description];
    self.adsArray = [NSArray array];
    [self.adsTableView reloadData];
}

#pragma mark - AppnextNativeAdOpenedDelegate

- (void) storeOpened:(AppnextAdData *)adData
{
    [self hideActivity];
    NSString *description = [NSString stringWithFormat:@"storeOpened: for - %@\r\n", adData.title];
    NSLog(@"%@", description);
    self.eventsTextView.text = [NSString stringWithFormat:@"%@%@", self.eventsTextView.text, description];
}

- (void) onError:(NSString *)error forAdData:(AppnextAdData *)adData
{
    [self hideActivity];
    NSString *description = [NSString stringWithFormat:@"onError:forAdData: - %@\r\n", error];
    NSLog(@"%@", description);
    self.eventsTextView.text = [NSString stringWithFormat:@"%@%@", self.eventsTextView.text, description];
    [self.view makeToast:description];
}

#pragma mark - AppnextNativeAdsTestTableViewCellDelegate

- (void) adClicked:(AppnextNativeAdsTestTableViewCell *)cell
{
    if (cell.adData)
    {
        NSLog(@"adClicked:%@", cell.adData.title);
        [self.api adClicked:cell.adData withAdOpenedDelegate:self];
        [self showActivity];
    }
}

- (void) impressionClicked:(AppnextNativeAdsTestTableViewCell *)cell
{
    if (cell.adData)
    {
        NSLog(@"impressionClicked:%@", cell.adData.title);
        [self.api adImpression:cell.adData];
    }
}

- (void) videoStartedClicked:(AppnextNativeAdsTestTableViewCell *)cell
{
    if (cell.adData)
    {
        NSLog(@"videoStartedClicked:%@", cell.adData.title);
        [self.api videoStarted:cell.adData];
    }
}

- (void) videoEndedClicked:(AppnextNativeAdsTestTableViewCell *)cell
{
    if (cell.adData)
    {
        NSLog(@"videoEndedClicked:%@", cell.adData.title);
        [self.api videoEnded:cell.adData];
    }
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.adsArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    AppnextNativeAdsTestTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"AdsCell"];
    NSUInteger row = indexPath.row;
    AppnextAdData *adData = nil;
    if (row < self.adsArray.count)
    {
        adData = [self.adsArray objectAtIndex:row];
    }
    cell.adData = adData;
    cell.delegate = self;
    [cell reloadViews];
    return cell;
}

#pragma mark - UITableViewDelegate

- (void) tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (cell && [cell isKindOfClass:[AppnextNativeAdsTestTableViewCell class]])
    {
        AppnextNativeAdsTestTableViewCell *nativeCell = (AppnextNativeAdsTestTableViewCell *)cell;
        if (nativeCell.adData)
        {
            [self.api adImpression:nativeCell.adData];
        }
    }
}

@end
